@@ -1,20 +1,16 @@
 package PDF::Template;
 
 use strict;
+use warnings;
 
-BEGIN {
-    use PDF::Template::Base;
-    use vars qw ($VERSION @ISA);
+use base 'PDF::Template::Base';
 
-    $VERSION = '0.22';
-    @ISA     = qw (PDF::Template::Base);
-}
+our $VERSION = '0.30';
 
-use pdflib_pl;
+use PDF::Writer;
 
-use File::Basename;
-use IO::File;
-use XML::Parser;
+use File::Basename qw( fileparse );
+use XML::Parser ();
 
 #-----------------------------------------------
 # TODO
@@ -27,8 +23,7 @@ use XML::Parser;
 #    sensible provider depending on what is installed.
 #-----------------------------------------------
 
-sub new
-{
+sub new {
     my $class = shift;
     my $self = $class->SUPER::new(@_);
 
@@ -36,27 +31,20 @@ sub new
     $self->{PARAM_MAP} = {} unless UNIVERSAL::isa($self->{PARAM_MAP}, 'HASH');
 
     $self->{PDF_VERSION} = 0;
-    for my $version (reverse 1 .. 6)
-    {
-        eval "UNIVERSAL::VERSION('pdflib_pl', $version.0)";
-        unless ($@)
-        {
-            $self->{PDF_VERSION} = $version;
-            last;
-        }
-    }
-    die "Cannot find pdflib_pl version",$/ unless $self->{PDF_VERSION};
 
     $self->_validate_option($_)
         for qw(OPENACTION OPENMODE);
 
-    $self->parse_xml($self->{FILENAME}) if defined $self->{FILENAME};
+    if ( !defined $self->{FILE} && defined $self->{FILENAME} ) {
+        $self->{FILE} = $self->{FILENAME};
+    }
+
+    $self->parse_xml($self->{FILE}) if defined $self->{FILE};
 
     return $self;
 }
 
-sub param
-{
+sub param {
     my $self = shift;
 
     # Allow an arbitrary number of hashrefs, so long as they're the first things
@@ -73,57 +61,35 @@ sub param
     return 1;
 }
 
-sub write_file
-{
+sub write_file {
     my $self = shift;
     my ($fname) = @_;
 
-    my $p = pdflib_pl::PDF_new();
-
-    pdflib_pl::PDF_open_file($p, $fname) == -1
-        && die "pdflib_pl::PDF_open_file could not open file '$fname'.", $/;
+    my $p = PDF::Writer->new;
+    $p->open($fname) or die "Could not open file '$fname'.", $/;
 
     $self->_prepare_output($p);
 
-    pdflib_pl::PDF_close($p);
+    $p->save();
 
     return 1;
 }
 
-sub output
-{
+sub get_buffer {
     my $self = shift;
 
-    $self->get_buffer(@_);
-}
-
-sub get_buffer
-{
-    my $self = shift;
-
-    my $p = pdflib_pl::PDF_new();
-
-    pdflib_pl::PDF_open_file($p, '') == -1
-        && die "pdflib_pl::PDF_open_file could not open buffer.", $/;
+    my $p = PDF::Writer->new;
+    $p->open() or die "Could not open buffer.", $/;
 
     $self->_prepare_output($p);
 
-    pdflib_pl::PDF_close($p);
-
-    return pdflib_pl::PDF_get_buffer($p);
+    return $p->stringify();
 }
+*output = \&get_buffer;
 
-sub parse
-{
+sub parse {
     my $self = shift;
-
-    $self->parse_xml(@_);
-}
-
-sub parse_xml
-{
-    my $self = shift;
-    my ($fname) = @_;
+    my ($file) = @_;
 
     my %Has_TextObject = map { $_ => undef } qw(
         BOOKMARK
@@ -131,22 +97,16 @@ sub parse_xml
         TEXTBOX
     );
 
-    my ($filename, $dirname) = fileparse($fname);
- 
     my @stack;
-    my $parser = XML::Parser->new(
-        Base => $dirname,
+    my @params = (
         Handlers => {
             Start => sub {
                 shift;
-#                { local $"="', '"; print "Start: '@_'\n"; }
                 my $name = uc shift;
 
                 # Pass the PDF encoding in.
-                if ($name eq 'PDFTEMPLATE')
-                {
-                    if (exists $self->{PDF_ENCODING})
-                    {
+                if ($name eq 'PDFTEMPLATE') {
+                    if (exists $self->{PDF_ENCODING}) {
                         push @_, (
                             PDF_ENCODING => $self->{PDF_ENCODING},
                         );
@@ -156,43 +116,34 @@ sub parse_xml
                 my $node = PDF::Template::Factory->create_node($name, @_);
                 die "'$name' (@_) didn't make a node!\n" unless defined $node;
 
-                if ($name eq 'VAR')
-                {
+                if ($name eq 'VAR') {
                     return unless @stack;
 
-                    if (exists $stack[-1]{TXTOBJ} && $stack[-1]{TXTOBJ}->isa('TEXTOBJECT'))
-                    {
+                    if (exists $stack[-1]{TXTOBJ} && $stack[-1]{TXTOBJ}->isa('TEXTOBJECT')) {
                         push @{$stack[-1]{TXTOBJ}{STACK}}, $node;
                     }
                 }
-                elsif ($name eq 'PDFTEMPLATE')
-                {
+                elsif ($name eq 'PDFTEMPLATE') {
                     push @{$self->{TEMPLATES}}, $node;
                 }
-                else
-                {
+                else {
                     push @{$stack[-1]{ELEMENTS}}, $node
                         if @stack;
                 }
 
                 push @stack, $node;
-#                print "Pushed $node onto stack\n";
             },
             Char => sub {
                 shift;
-#                { local $"="', '"; print "Char:  '@_'\n"; }
                 return unless @stack;
 
                 my $parent = $stack[-1];
-                if (exists $parent->{TXTOBJ} && $parent->{TXTOBJ}->isa('TEXTOBJECT'))
-                {
-#                    print "Added '@_' to TextObject stack for '$parent'\n";
+                if (exists $parent->{TXTOBJ} && $parent->{TXTOBJ}->isa('TEXTOBJECT')) {
                     push @{$parent->{TXTOBJ}{STACK}}, @_;
                 }
             },
             End => sub {
                 shift;
-#                { local $"="', '"; print "End:   '@_'\n"; }
                 return unless @stack;
 
                 pop @stack if $stack[-1]->isa(uc $_[0]);
@@ -200,49 +151,54 @@ sub parse_xml
         },
     );
 
-    {
-        my $fh = IO::File->new($fname)
-            || die "Cannot open '$fname' for reading: $!\n";
- 
-        $parser->parse(do { local $/ = undef; <$fh> });
- 
-        $fh->close;
+    if ( exists $self->{PDF_ENCODING} ) {
+        push @params, ProtocolEncoding => $self->{PDF_ENCODING};
     }
 
+    if ( ref $file ) {
+        *INFILE = $file;
+    }
+    else {
+        my ($filename, $dirname) = fileparse($file);
+        push @params, Base => $dirname;
+
+        open( INFILE, '<', $file )
+            || die "Cannot open '$file' for reading: $!\n";
+    }
+
+    my $parser = XML::Parser->new( @params );
+    $parser->parse(do { local $/ = undef; <INFILE> });
+
+    close INFILE
+        unless ref $file;
+
     return 1;
 }
+*parse_xml = \&parse;
 
-my %NoSetProperty = (
-    'CreationDate' => 1,
-    'Producer' => 1,
-    'ModDate' => 1,
-    'Trapped' => 1,
+my %NoSetProperty = map { $_ => 1 } qw(
+    CreationDate Producer ModDate Trapped
 );
 
-sub _prepare_output
-{
+sub _prepare_output {
     my $self = shift;
     my ($p) = @_;
 
-    pdflib_pl::PDF_set_parameter($p, 'openaction', $self->{OPENACTION});
-    pdflib_pl::PDF_set_parameter($p, 'openmode', $self->{OPENMODE});
+    $p->parameter('openaction' => $self->{OPENACTION});
+    $p->parameter('openmode' => $self->{OPENMODE});
 
-    if (UNIVERSAL::isa($self->{INFO}, 'HASH'))
-    {
-        foreach my $key ( keys %{$self->{INFO}} )
-        {
-            if ($NoSetProperty{$key})
-            {
+    if (UNIVERSAL::isa($self->{INFO}, 'HASH')) {
+        foreach my $key ( keys %{$self->{INFO}} ) {
+            if ($NoSetProperty{$key}) {
                 warn "Document property '$key' cannot be set.", $/;
                 next;
             }
 
-            pdflib_pl::PDF_set_info($p, $key, $self->{INFO}{$key});
+            $p->info($key, $self->{INFO}{$key});
         }
     }
-    else
-    {
-        pdflib_pl::PDF_set_info($p, $_, __PACKAGE__) for qw/Creator Author/;
+    else {
+        $p->info($_, __PACKAGE__) for qw/Creator Author/;
     }
 
     # __PAGE__ is incremented after the page is done.
@@ -266,7 +222,7 @@ sub _prepare_output
     );
 
     # Do a first pass through, noting important values
-    $_->preprocess($context) for @{$self->{TEMPLATES}};
+#    $_->preprocess($context) for @{$self->{TEMPLATES}};
 
     # Do a second pass through, for actual rendering
     $_->render($context) for @{$self->{TEMPLATES}};
@@ -289,9 +245,9 @@ PDF::Template - PDF::Template
 
   use PDF::Template
 
-  my $pdf = PDF::Template->new(
-     filename => 'some_template.xml',
-  );
+  my $pdf = PDF::Template->new({
+     file => 'some_template.xml',
+  });
 
   $pdf->param(%my_params);
 
@@ -302,15 +258,15 @@ PDF::Template - PDF::Template
 =head1 DESCRIPTION
 
 PDF::Template is a PDF layout system that uses the same data structures as
-HTML::Template.
+L<HTML::Template>.
 
 =head1 OVERVIEW
 
 PDF::Template is a PDF layout system that uses the same data structures as
-HTML::Template. Unlike HTML::Template, this is a full layout system. This means
+L<HTML::Template>. Unlike L<HTML::Template>, this is a full layout system. This means
 you will have to describe where each item will be on the page. (This is in
-contrast to HTML::Template, which adds on to HTML. The layout is determined by
-the HTML, not HTML::Template.)
+contrast to L<HTML::Template>, which adds on to L<HTML::Template>ut is determined by
+the HTML, not L<HTML::Template>.)
 
 PDF::Template uses an XML document as the template. However, the XML is not
 completely compliant. The only difference (that I'm aware of) is that any node
@@ -326,6 +282,124 @@ the value of a parameter and it doesn't have that value as an attribute, it will
 ask its parent for the value. If the parent doesn't have it, it will ask its
 parent, and so on.
 
+=head1 METHODS
+
+=over 4
+
+=item * C<new( [$opts] )>
+
+This will create a new instance of PDF::Template. $opts is an optional hashref
+that can contain the following parameters:
+
+=over 4
+
+=item * file
+
+This is either the name of the file or the filehandle of the open file. If it
+is present, C<parse()> will be called upon that filename/filehandle. Otherwise,
+after new() is called, you will have to call C<parse()> yourself.
+
+filename is a synonym for file.
+
+=item * openaction
+
+This is the action that the PDF reader will take when it opens this file. The
+valid values are:
+
+=over 4
+
+=item * fitbox
+
+=item * fitheight
+
+=item * fitpage (default)
+
+=item * fitwidth
+
+=item * retain
+
+=back
+
+=item * openmode
+
+This is the mode that the PDF reader will use when it opens this file. The
+valid values are:
+
+=over 4
+
+=item * bookmarks
+
+=item * fullscreen
+
+=item * none (default)
+
+=item * thumbnails
+
+=back
+
+=item * info
+
+This is a hashref of information that you wish to have the PDF retain as
+metadata. If this is not present, both Author and Creator will be set to
+PDF::Template.
+
+The following keys are not supported:
+
+=over 4
+
+=item * CreationDate
+
+=item * Producer
+
+=item * ModDate
+
+=item * Trapped
+
+=back
+
+=item * pdf_encoding
+
+This is the encoding that the template is in. It defaults to the host
+encoding. This is different from the encoding parameter for the pdftemplate
+tag.
+
+=back
+
+=item * C<parse( $file )>
+
+This will parse the XML template into the appropriate datastructure(s) needed
+for PDF::Template to function.
+
+=item * C<parse_xml( $file )>
+
+This is a deprecated synonym for C<parse()>.
+
+=item * C<param( key => value, [ key => value, ... ] )>
+
+This will set the parameters that PDF::Template will use to merge the template
+with. This method is identical to the HTML::Template or Template Toolkit
+method of the same name.
+
+=item * C<write_file( $filename )>
+
+This will write the rendered PDF to the file specified in $filename.
+
+=item * C<get_buffer()>
+
+This will return the rendered PDF stringified in a form appropriate for
+returning over an HTTP connection.
+
+=item * C<output()>
+
+This is a synonym for C<get_buffer()> provided for HTML::Template
+compatibility.
+
+=item * C<register( ... )>
+
+XXX
+
+=back
+
 =head1 USAGE
 
 There are a few consistency rules that that every PDF::Template has to follow:
@@ -341,19 +415,20 @@ child of the PDFTEMPLATE node)
 
 =item 4 There must be a FONT tag as an ancestor of every TEXTBOX node
 
-=item 5 Within a PAGEDEF, there can only be on HEADER node and one FOOTER node
+=item 5 Within a PAGEDEF, there can only be one HEADER node and one FOOTER node
 
-=back 4
+=back
 
 For more information about each node, please see the POD for that class.
 
 =head1 WWW CAVEATS
 
-When taking an HTML page and adding a PDF option, there are a few differences totake into account. The primary one is the idea of pagebreaks. HTML is displayed
-as a single page, with scrolling. Paper doesn't scroll, so when there should be
-a new page is something PDF::Template works very hard at determining. It will
-take into account any header and footer information you've provided, as well as
-page sizes.
+When taking an HTML page and adding a PDF option, there are a few differences
+to take into account. The primary one is the idea of pagebreaks. HTML is
+displayed as a single page, with scrolling. Paper doesn't scroll, so when
+there should be a new page is something PDF::Template works very hard at
+determining. It will take into account any header and footer information
+you've provided, as well as page sizes.
 
 The second is that you have to determine how wide you want your text to be. One
 of the most common activities is to take a tabular report and covert it to a
@@ -369,18 +444,19 @@ None, that I'm aware of.
 
 =head1 LIMITATIONS
 
-Currently, the only PDF rendered PDF::Template supports is PDFLib (available at
-www.pdflib.com). An upcoming release of PDF::Template will also support
-PDF::API2. Unless you need Unicode support, PDFLib Lite is sufficient (and
-free). Please see www.pdflib.com for more details.
+Currently, the only PDF renderer PDF::Template supports is PDFlib (available at
+www.pdflib.com). The next release of PDF::Template will also support PDF::API2.
+Unless you need Unicode support, PDFlib Lite is sufficient (and free). Please
+see L<http://www.pdflib.com> for more details.
 
-I am aware that PDFLib will not compile under AIX or Cygwin. These are problems
-that PDFLib has acknowledged to me.
+I am aware that PDFlib will not compile under AIX or Cygwin. These are
+problems that PDFlib has acknowledged to me.
 
-=head1 AUTHOR
+=head1 AUTHOR/MAINTAINER
 
 Originally written by Dave Ferrance (dave@ferrance.org)
-Taken over after v0.05 by Rob Kinyon (rob.kinyon@gmail.com)
+
+Taken over after v0.05 by Rob Kinyon (rob.kinyon@iinteractive.com)
 
 =head1 CONTRIBUTORS
 
@@ -388,22 +464,29 @@ Patches and ideas provided by:
 
 =over 4
 
+=item * Autrijus Tang
+
+Provided the impetus to move to L<PDF::Writer> (which he also wrote).
+
 =item * Michael Kiwala
 
+Aided in the design and testing of the transition from Dave Ferrance's
+version.
+
 =item * Nathan Byrd
 
-=back 4
+Provided nearly all the initial doublebyte expertise.
 
-Additionally, there is a mailing list at http://groups-beta.google.com/group/PDFTemplate
+=back
+
+Additionally, there is a mailing list at
+L<http://groups.google.com/group/PDFTemplate>
 
 =head1 COPYRIGHT
 
 This program is free software; you can redistribute
 it and/or modify it under the same terms as Perl itself.
 
-The full text of the license can be found in the
-LICENSE file included with this module.
-
 =head1 SEE ALSO
 
 perl(1).
